@@ -1,4 +1,5 @@
 use self::static_check::should_use_create_element;
+use crate::refresh::options::{deserialize_refresh, RefreshOptions};
 use dashmap::DashMap;
 use once_cell::sync::Lazy;
 use regex::Regex;
@@ -7,11 +8,11 @@ use std::{iter, iter::once, mem};
 use string_enum::StringEnum;
 use swc_atoms::{js_word, JsWord};
 use swc_common::{
-    comments::{CommentKind, Comments},
+    comments::{Comment, CommentKind, Comments},
     iter::IdentifyLast,
     sync::Lrc,
     util::take::Take,
-    FileName, SourceMap, Spanned, DUMMY_SP,
+    FileName, Mark, SourceMap, Span, Spanned, DUMMY_SP,
 };
 use swc_ecma_ast::*;
 use swc_ecma_parser::{Parser, StringInput, Syntax};
@@ -21,8 +22,6 @@ use swc_ecma_utils::{
 };
 use swc_ecma_visit::{as_folder, noop_visit_mut_type, Fold, VisitMut, VisitMutWith};
 
-use crate::refresh::options::{deserialize_refresh, RefreshOptions};
-
 mod static_check;
 #[cfg(test)]
 mod tests;
@@ -116,15 +115,21 @@ fn default_throw_if_namespace() -> bool {
     true
 }
 
-fn parse_classic_option(cm: &SourceMap, name: &str, src: String) -> Box<Expr> {
-    static CACHE: Lazy<DashMap<String, Box<Expr>>> = Lazy::new(|| DashMap::with_capacity(2));
+fn parse_classic_option(
+    cm: &SourceMap,
+    name: &str,
+    src: String,
+    top_level_mark: Mark,
+) -> Box<Expr> {
+    static CACHE: Lazy<DashMap<(String, Mark), Box<Expr>, ahash::RandomState>> =
+        Lazy::new(|| DashMap::with_capacity_and_hasher(2, Default::default()));
 
     let fm = cm.new_source_file(FileName::Custom(format!("<jsx-config-{}.js>", name)), src);
-    if let Some(expr) = CACHE.get(&**fm.src) {
+    if let Some(expr) = CACHE.get(&((*fm.src).clone(), top_level_mark)) {
         return expr.clone();
     }
 
-    let expr = Parser::new(Syntax::default(), StringInput::from(&*fm), None)
+    let mut expr = Parser::new(Syntax::default(), StringInput::from(&*fm), None)
         .parse_expr()
         .map_err(|e| {
             if HANDLER.is_set() {
@@ -139,20 +144,46 @@ fn parse_classic_option(cm: &SourceMap, name: &str, src: String) -> Box<Expr> {
             )
         });
 
-    CACHE.insert((*fm.src).clone(), expr.clone());
+    apply_mark(&mut expr, top_level_mark);
+    CACHE.insert(((*fm.src).clone(), top_level_mark), expr.clone());
 
     expr
 }
 
+fn apply_mark(e: &mut Expr, mark: Mark) {
+    match e {
+        Expr::Ident(i) => {
+            i.span = i.span.apply_mark(mark);
+        }
+        Expr::Member(MemberExpr {
+            obj: ExprOrSuper::Expr(obj),
+            ..
+        }) => {
+            apply_mark(&mut **obj, mark);
+        }
+        _ => {}
+    }
+}
+
 /// `@babel/plugin-transform-react-jsx`
 ///
 /// Turn JSX into React function calls
-pub fn jsx<C>(cm: Lrc<SourceMap>, comments: Option<C>, options: Options) -> impl Fold + VisitMut
+///
+///
+/// `top_level_mark` should be [Mark] passed to
+/// [swc_ecma_transforms_base::resolver::resolver_with_mark].
+pub fn jsx<C>(
+    cm: Lrc<SourceMap>,
+    comments: Option<C>,
+    options: Options,
+    top_level_mark: Mark,
+) -> impl Fold + VisitMut
 where
     C: Comments,
 {
     as_folder(Jsx {
         cm: cm.clone(),
+        top_level_mark,
         next: options.next,
         runtime: options.runtime.unwrap_or_default(),
         import_source: options.import_source.into(),
@@ -161,11 +192,16 @@ where
         import_fragment: None,
         import_create_element: None,
 
-        pragma: ExprOrSuper::Expr(parse_classic_option(&cm, "pragma", options.pragma)),
+        pragma: ExprOrSuper::Expr(parse_classic_option(
+            &cm,
+            "pragma",
+            options.pragma,
+            top_level_mark,
+        )),
         comments,
         pragma_frag: ExprOrSpread {
             spread: None,
-            expr: parse_classic_option(&cm, "pragmaFrag", options.pragma_frag),
+            expr: parse_classic_option(&cm, "pragmaFrag", options.pragma_frag, top_level_mark),
         },
         use_builtins: options.use_builtins,
         use_spread: options.use_spread,
@@ -180,6 +216,8 @@ where
 {
     cm: Lrc<SourceMap>,
 
+    top_level_mark: Mark,
+
     next: bool,
     runtime: Runtime,
     /// For automatic runtime.
@@ -691,91 +729,18 @@ where
     }
 }
 
-impl<C> VisitMut for Jsx<C>
+impl<C> Jsx<C>
 where
     C: Comments,
 {
-    noop_visit_mut_type!();
+    fn parse_directives(&mut self, span: Span) -> bool {
+        let mut found = false;
 
-    fn visit_mut_module(&mut self, module: &mut Module) {
         let leading = if let Some(comments) = &self.comments {
-            let leading = comments.take_leading(module.span.lo);
+            let leading = comments.take_leading(span.lo);
 
             if let Some(leading) = &leading {
-                for leading in &**leading {
-                    if leading.kind != CommentKind::Block {
-                        continue;
-                    }
-
-                    for line in leading.text.lines() {
-                        let mut line = line.trim();
-                        if line.starts_with('*') {
-                            line = line[1..].trim();
-                        }
-
-                        if !line.starts_with("@jsx") {
-                            continue;
-                        }
-
-                        if line.starts_with("@jsxRuntime ") {
-                            let src = line.replace("@jsxRuntime ", "").trim().to_string();
-                            if src == "classic" {
-                                self.runtime = Runtime::Classic;
-                            } else if src == "automatic" {
-                                self.runtime = Runtime::Automatic;
-                            } else {
-                                todo!("proper error reporting for wrong `@jsxRuntime`")
-                            }
-                            continue;
-                        }
-
-                        if line.starts_with("@jsxImportSource") {
-                            let src = line.replace("@jsxImportSource", "").trim().to_string();
-                            self.runtime = Runtime::Automatic;
-                            self.import_source = src.into();
-                        }
-
-                        if line.starts_with("@jsxFrag") {
-                            if self.runtime == Runtime::Automatic {
-                                HANDLER.with(|handler| {
-                                    handler
-                                        .struct_span_err(
-                                            module.span,
-                                            "pragma and pragmaFrag cannot be set when runtime is \
-                                             automatic",
-                                        )
-                                        .emit()
-                                });
-                            }
-
-                            let src = line.replace("@jsxFrag", "").trim().to_string();
-                            self.pragma_frag = ExprOrSpread {
-                                expr: parse_classic_option(&self.cm, "module-jsx-pragma-frag", src),
-                                spread: None,
-                            };
-                        } else if line.starts_with("@jsx ") {
-                            if self.runtime == Runtime::Automatic {
-                                HANDLER.with(|handler| {
-                                    handler
-                                        .struct_span_err(
-                                            module.span,
-                                            "pragma and pragmaFrag cannot be set when runtime is \
-                                             automatic",
-                                        )
-                                        .emit()
-                                });
-                            }
-
-                            let src = line.replace("@jsx", "").trim().to_string();
-
-                            self.pragma = ExprOrSuper::Expr(parse_classic_option(
-                                &self.cm,
-                                "module-jsx-pragma",
-                                src,
-                            ));
-                        }
-                    }
-                }
+                found |= self.parse_comment_contents(span, &leading);
             }
 
             leading
@@ -785,7 +750,161 @@ where
 
         if let Some(leading) = leading {
             if let Some(comments) = &self.comments {
-                comments.add_leading_comments(module.span.lo, leading);
+                comments.add_leading_comments(span.lo, leading);
+            }
+        }
+
+        found
+    }
+
+    /// If we found required jsx directives, we returns true.
+    fn parse_comment_contents(&mut self, span: Span, leading: &[Comment]) -> bool {
+        let mut found = false;
+
+        for cmt in leading {
+            if cmt.kind != CommentKind::Block {
+                continue;
+            }
+
+            for line in cmt.text.lines() {
+                let mut line = line.trim();
+                if line.starts_with('*') {
+                    line = line[1..].trim();
+                }
+
+                if !line.starts_with("@jsx") {
+                    continue;
+                }
+
+                if line.starts_with("@jsxRuntime ") {
+                    let src = line.replace("@jsxRuntime ", "").trim().to_string();
+                    found = true;
+                    if src == "classic" {
+                        self.runtime = Runtime::Classic;
+                    } else if src == "automatic" {
+                        self.runtime = Runtime::Automatic;
+                    } else {
+                        todo!("proper error reporting for wrong `@jsxRuntime`")
+                    }
+                    continue;
+                }
+
+                if line.starts_with("@jsxImportSource") {
+                    let src = line.replace("@jsxImportSource", "").trim().to_string();
+                    self.runtime = Runtime::Automatic;
+                    self.import_source = src.into();
+                    found = true;
+                }
+
+                if line.starts_with("@jsxFrag") {
+                    found = true;
+
+                    if self.runtime == Runtime::Automatic {
+                        HANDLER.with(|handler| {
+                            handler
+                                .struct_span_err(
+                                    span,
+                                    "pragma and pragmaFrag cannot be set when runtime is automatic",
+                                )
+                                .emit()
+                        });
+                    }
+
+                    let src = line.replace("@jsxFrag", "").trim().to_string();
+                    self.pragma_frag = ExprOrSpread {
+                        expr: parse_classic_option(
+                            &self.cm,
+                            "module-jsx-pragma-frag",
+                            src,
+                            self.top_level_mark,
+                        ),
+                        spread: None,
+                    };
+                } else if line.starts_with("@jsx ") {
+                    found = true;
+
+                    if self.runtime == Runtime::Automatic {
+                        HANDLER.with(|handler| {
+                            handler
+                                .struct_span_err(
+                                    span,
+                                    "pragma and pragmaFrag cannot be set when runtime is automatic",
+                                )
+                                .emit()
+                        });
+                    }
+
+                    let src = line.replace("@jsx", "").trim().to_string();
+
+                    self.pragma = ExprOrSuper::Expr(parse_classic_option(
+                        &self.cm,
+                        "module-jsx-pragma",
+                        src,
+                        self.top_level_mark,
+                    ));
+                }
+            }
+        }
+
+        found
+    }
+}
+
+impl<C> VisitMut for Jsx<C>
+where
+    C: Comments,
+{
+    noop_visit_mut_type!();
+
+    fn visit_mut_expr(&mut self, expr: &mut Expr) {
+        let top_level_node = self.top_level_node;
+        let mut did_work = false;
+
+        if let Expr::JSXElement(el) = expr {
+            did_work = true;
+            // <div></div> => React.createElement('div', null);
+            *expr = self.jsx_elem_to_expr(*el.take());
+        } else if let Expr::JSXFragment(frag) = expr {
+            // <></> => React.createElement(React.Fragment, null);
+            did_work = true;
+            *expr = self.jsx_frag_to_expr(frag.take());
+        } else if let Expr::Paren(ParenExpr {
+            expr: inner_expr, ..
+        }) = expr
+        {
+            if let Expr::JSXElement(el) = &mut **inner_expr {
+                did_work = true;
+                *expr = self.jsx_elem_to_expr(*el.take());
+            } else if let Expr::JSXFragment(frag) = &mut **inner_expr {
+                // <></> => React.createElement(React.Fragment, null);
+                did_work = true;
+                *expr = self.jsx_frag_to_expr(frag.take());
+            }
+        }
+
+        if did_work {
+            self.top_level_node = false;
+        }
+
+        expr.visit_mut_children_with(self);
+
+        self.top_level_node = top_level_node;
+    }
+
+    fn visit_mut_member_expr(&mut self, e: &mut MemberExpr) {
+        e.obj.visit_mut_with(self);
+        if e.computed {
+            e.prop.visit_mut_with(self);
+        }
+    }
+
+    fn visit_mut_module(&mut self, module: &mut Module) {
+        self.parse_directives(module.span);
+
+        for item in &module.body {
+            let span = item.span();
+            if self.parse_directives(span) {
+                break;
             }
         }
 
@@ -864,48 +983,6 @@ where
             }
         }
     }
-
-    fn visit_mut_expr(&mut self, expr: &mut Expr) {
-        let top_level_node = self.top_level_node;
-        let mut did_work = false;
-
-        if let Expr::JSXElement(el) = expr {
-            did_work = true;
-            // <div></div> => React.createElement('div', null);
-            *expr = self.jsx_elem_to_expr(*el.take());
-        } else if let Expr::JSXFragment(frag) = expr {
-            // <></> => React.createElement(React.Fragment, null);
-            did_work = true;
-            *expr = self.jsx_frag_to_expr(frag.take());
-        } else if let Expr::Paren(ParenExpr {
-            expr: inner_expr, ..
-        }) = expr
-        {
-            if let Expr::JSXElement(el) = &mut **inner_expr {
-                did_work = true;
-                *expr = self.jsx_elem_to_expr(*el.take());
-            } else if let Expr::JSXFragment(frag) = &mut **inner_expr {
-                // <></> => React.createElement(React.Fragment, null);
-                did_work = true;
-                *expr = self.jsx_frag_to_expr(frag.take());
-            }
-        }
-
-        if did_work {
-            self.top_level_node = false;
-        }
-
-        expr.visit_mut_children_with(self);
-
-        self.top_level_node = top_level_node;
-    }
-
-    fn visit_mut_member_expr(&mut self, e: &mut MemberExpr) {
-        e.obj.visit_mut_with(self);
-        if e.computed {
-            e.prop.visit_mut_with(self);
-        }
-    }
 }
 
 impl<C> Jsx<C>
