@@ -7,7 +7,7 @@ pub use self::{
     refresh::{options::RefreshOptions, refresh},
 };
 use std::mem;
-use swc_common::{chain, comments::Comments, sync::Lrc, SourceMap};
+use swc_common::{chain, comments::Comments, sync::Lrc, Mark, SourceMap};
 use swc_ecma_visit::Fold;
 
 mod display_name;
@@ -20,7 +20,16 @@ mod refresh;
 /// `@babel/preset-react`
 ///
 /// Preset for all React plugins.
-pub fn react<C>(cm: Lrc<SourceMap>, comments: Option<C>, mut options: Options) -> impl Fold
+///
+///
+/// `top_level_mark` should be [Mark] passed to
+/// [swc_ecma_transforms_base::resolver::resolver_with_mark].
+pub fn react<C>(
+    cm: Lrc<SourceMap>,
+    comments: Option<C>,
+    mut options: Options,
+    top_level_mark: Mark,
+) -> impl Fold
 where
     C: Comments + Clone,
 {
@@ -32,7 +41,7 @@ where
         jsx_src(development, cm.clone()),
         jsx_self(development),
         refresh(development, refresh_options, cm.clone(), comments.clone()),
-        jsx(cm.clone(), comments.clone(), options),
+        jsx(cm.clone(), comments.clone(), options, top_level_mark),
         display_name(),
         pure_annotations(comments),
     )
